import * as React from 'react';
import { scaleTime } from 'd3-scale';
import { Slider, Rail, Handles, Tracks, Ticks } from 'react-compound-slider';
import { format, addHours, startOfToday, endOfToday, differenceInMilliseconds, isBefore, isAfter, set, addMinutes } from 'date-fns';
import SliderRail from './components/SliderRail';
import Track from './components/Track';
import Tick from './components/Tick';
import Handle from './components/Handle';
import './styles/index.scss';
const getTimelineConfig = (timelineStart, timelineLength) => date => {
    const percent = (differenceInMilliseconds(date, timelineStart) / timelineLength) * 100;
    const value = Number(format(date, 'T'));
    return { percent, value };
};
const getFormattedBlockedIntervals = (blockedDates = [], [startTime, endTime]) => {
    if (!blockedDates.length) {
        return null;
    }
    const timelineLength = differenceInMilliseconds(endTime, startTime);
    const getConfig = getTimelineConfig(startTime, timelineLength);
    const formattedBlockedDates = blockedDates.map((interval, index) => {
        let { start, end } = interval;
        if (isBefore(start, startTime)) {
            start = startTime;
        }
        if (isAfter(end, endTime)) {
            end = endTime;
        }
        const source = getConfig(start);
        const target = getConfig(end);
        return { id: `blocked-track-${index}`, source, target };
    });
    return formattedBlockedDates;
};
const getNowConfig = ([startTime, endTime]) => {
    const timelineLength = differenceInMilliseconds(endTime, startTime);
    const getConfig = getTimelineConfig(startTime, timelineLength);
    const source = getConfig(new Date());
    const target = getConfig(addMinutes(new Date(), 1));
    return { id: 'now-track', source, target };
};
export function TimeRange(props) {
    var _a;
    const getDisabledIntervals = () => {
        return getFormattedBlockedIntervals(props.disabledIntervals, props.timelineInterval);
    };
    const getNow = () => {
        return getNowConfig(props.timelineInterval);
    };
    const onChange = (newTime) => {
        const formattedNewTime = [new Date(newTime[0]), new Date(newTime[1])];
        props.onChangeCallback(formattedNewTime);
    };
    const checkIsSelectedIntervalNotValid = ([start, end], source, target) => {
        const { value: startInterval } = source;
        const { value: endInterval } = target;
        if ((startInterval > start && endInterval <= end) || (startInterval >= start && endInterval < end)) {
            return true;
        }
        if (start >= startInterval && end <= endInterval) {
            return true;
        }
        const isStartInBlockedInterval = start > startInterval && start < endInterval && end >= endInterval;
        const isEndInBlockedInterval = end < endInterval && end > startInterval && start <= startInterval;
        return isStartInBlockedInterval || isEndInBlockedInterval;
    };
    const onUpdate = newTime => {
        const { onUpdateCallback } = props;
        const disabledIntervals = getDisabledIntervals();
        if (disabledIntervals === null || disabledIntervals === void 0 ? void 0 : disabledIntervals.length) {
            const isValuesNotValid = disabledIntervals.some(({ source, target }) => checkIsSelectedIntervalNotValid(newTime, source, target));
            onUpdateCallback({ error: isValuesNotValid });
            return;
        }
        onUpdateCallback({ error: false });
    };
    const getDateTicks = () => {
        const { timelineInterval, ticksNumber } = props;
        return scaleTime()
            .domain(timelineInterval)
            .ticks(ticksNumber)
            .map(t => +t);
    };
    const { sliderRailClassName, timelineInterval, selectedInterval, containerClassName, error, step, showNow, formatTick, mode } = props;
    const domain = timelineInterval.map(t => Number(t));
    return (React.createElement("div", { className: containerClassName || 'react_time_range__time_range_container' },
        React.createElement(Slider, { mode: mode, step: step, domain: domain, onUpdate: onUpdate, onChange: onChange, values: (selectedInterval === null || selectedInterval === void 0 ? void 0 : selectedInterval.map(t => +t)) || [], rootStyle: { position: 'relative', width: '100%' } },
            React.createElement(Rail, null, ({ getRailProps }) => React.createElement(SliderRail, { className: sliderRailClassName, getRailProps: getRailProps })),
            React.createElement(Handles, null, ({ handles, getHandleProps }) => (React.createElement(React.Fragment, null, handles.map(handle => (React.createElement(Handle, { error: error, key: handle.id, handle: handle, domain: [domain[0], domain[1]], getHandleProps: getHandleProps })))))),
            React.createElement(Tracks, { left: false, right: false }, ({ tracks, getTrackProps }) => (React.createElement(React.Fragment, null, tracks === null || tracks === void 0 ? void 0 : tracks.map(({ id, source, target }) => (React.createElement(Track, { error: error, key: id, source: source, target: target, getTrackProps: getTrackProps })))))),
            ((_a = getDisabledIntervals()) === null || _a === void 0 ? void 0 : _a.length) && (React.createElement(Tracks, { left: false, right: false }, ({ getTrackProps }) => {
                var _a;
                return (React.createElement(React.Fragment, null, (_a = getDisabledIntervals()) === null || _a === void 0 ? void 0 : _a.map(({ id, source, target }) => (React.createElement(Track, { key: id, source: source, target: target, getTrackProps: getTrackProps, disabled: true })))));
            })),
            showNow && (React.createElement(Tracks, { left: false, right: false }, ({ getTrackProps }) => {
                var _a, _b, _c;
                return (React.createElement(Track, { key: (_a = getNow()) === null || _a === void 0 ? void 0 : _a.id, source: (_b = getNow()) === null || _b === void 0 ? void 0 : _b.source, target: (_c = getNow()) === null || _c === void 0 ? void 0 : _c.target, getTrackProps: getTrackProps }));
            })),
            React.createElement(Ticks, { values: getDateTicks() }, ({ ticks }) => (React.createElement(React.Fragment, null, ticks.map(tick => (React.createElement(Tick, { key: tick.id, tick: tick, count: ticks.length, format: formatTick })))))))));
}
TimeRange.defaultProps = {
    selectedInterval: [
        set(new Date(), { minutes: 0, seconds: 0, milliseconds: 0 }),
        set(addHours(new Date(), 1), { minutes: 0, seconds: 0, milliseconds: 0 })
    ],
    timelineInterval: [startOfToday(), endOfToday()],
    formatTick: ms => format(new Date(ms), 'HH:mm'),
    disabledIntervals: [],
    step: 1000 * 60 * 30,
    ticksNumber: 48,
    error: false,
    mode: 3
};
export default TimeRange;
//# sourceMappingURL=TimeRange.js.map